@@ -19,6 +19,7 @@
 
 #include <string.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 #include "gailwidget.h"
 #include "gailnotebookpage.h"
 #include "gailcanvaswidget.h"
@@ -80,14 +81,6 @@ static gboolean gail_widget_set_size       (AtkComponent    *component,
                                             gint            width,
                                             gint            height);
 
-#if 0
-/*
- * We will get the parent from the widget structure rather than
- * use this function
- */
-static GtkWidget* _gail_widget_get_parent        (GtkWidget     *widget);
-#endif
-
 static gint       gail_widget_map_gtk            (GtkWidget     *widget);
 static void       gail_widget_real_notify_gtk    (GObject       *obj,
                                                   GParamSpec    *pspec);
@@ -475,6 +468,22 @@ gail_widget_ref_relation_set (AtkObject *obj)
                     }
                 }
             }
+          else if (GTK_IS_COMBO_BOX (widget))
+            /*
+             * Handle the case when GtkFileChooserButton is the mnemonic
+             * widget.  The GtkComboBox which is a child of the
+             * GtkFileChooserButton should be the mnemonic widget.
+             * See bug #359843.
+             */
+            {
+              GtkWidget *temp_widget;
+
+              temp_widget = gtk_widget_get_parent (widget);
+              if (GTK_IS_HBOX (temp_widget))
+                {
+                  label = find_label (temp_widget);
+                }
+            }
         }
 
       if (label)
@@ -533,16 +542,13 @@ gail_widget_ref_state_set (AtkObject *accessible)
        * GailWidget data structure so we can determine whether the value has 
        * changed.
        */
-      if (gail_widget_on_screen (widget))
+      if (GTK_WIDGET_VISIBLE (widget))
         {
-          if (GTK_WIDGET_VISIBLE (widget))
+          atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
+          if (gail_widget_on_screen (widget) &&
+              GTK_WIDGET_MAPPED (widget))
             {
-              atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
-
-              if (GTK_WIDGET_MAPPED (widget))
-                {
-                  atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
-                }
+              atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
             }
         }
   
@@ -761,7 +767,7 @@ gail_widget_grab_focus (AtkComponent   *component)
       gtk_widget_grab_focus (widget);
       toplevel = gtk_widget_get_toplevel (widget);
       if (GTK_WIDGET_TOPLEVEL (toplevel))
-        gtk_window_present (GTK_WINDOW (toplevel)); 
+        gtk_window_present_with_time (GTK_WINDOW (toplevel), gdk_x11_get_server_time (widget->window));
       return TRUE;
     }
   else
@@ -886,26 +892,6 @@ gail_widget_set_size (AtkComponent   *component,
    return FALSE;
 }
 
-#if 0
-static GtkWidget
-_gail_widget_get_parent (GtkWidget* widget)
-{
-  GtkWidget *parent_widget;
-  gpointer pointer;
-  GValue value = { 0, };
-
-  g_return_val_if_fail (GTK_WIDGET (widget), NULL);
-
-  g_value_init (&value, G_TYPE_OBJECT);
-  g_object_get_property (G_OBJECT (widget), "parent", &value);
-  pointer = g_value_get_object (&value);
- 
-  g_return_val_if_fail (GTK_IS_WIDGET (pointer), NULL);
-
-  return GTK_WIDGET (pointer);
-}
-#endif
-
 /*
  * This function is a signal handler for notify_in_event and focus_out_event
  * signal which gets emitted on a GtkWidget.
